@@ -1,5 +1,5 @@
 package Perl::MinimumVersion;
-
+$Perl::MinimumVersion::VERSION = '1.38_01';
 =pod
 
 =head1 NAME
@@ -12,7 +12,7 @@ Perl::MinimumVersion - Find a minimum required version of perl for Perl code
   $object = Perl::MinimumVersion->new( $filename );
   $object = Perl::MinimumVersion->new( \$source  );
   $object = Perl::MinimumVersion->new( $ppi_document );
-  
+
   # Find the minimum version
   $version = $object->minimum_version;
 
@@ -30,53 +30,66 @@ Future plans are to also add support for tracing module dependencies.
 Using C<Perl::MinimumVersion> is dead simple, the synopsis pretty much
 covers it.
 
+The distribution comes with a script called L<perlver>,
+which is the easiest way to run C<Perl::MinimumVersion> on your code:
+
+ % perlver lib/Foo/Bar.pm
+
+See the L<documentation for perlver|perlver> for more details.
+
 =head1 METHODS
 
 =cut
 
 use 5.006;
 use strict;
-use version      ();
-use Carp         ();
-use Exporter     ();
-use List::Util   ();
-use Params::Util ('_INSTANCE', '_CLASS');
-use PPI::Util    ('_Document');
-use PPI          ();
-use Perl::Critic::Utils 1.104 qw{
+use warnings;
+use version             0.76   ();
+use Carp                        ();
+use Exporter                    ();
+use List::Util          1.20    qw(max first);
+use Params::Util        0.25    ('_INSTANCE', '_CLASS');
+use PPI::Util                   ('_Document');
+use PPI                 1.215   ();
+use Perl::Critic::Utils 1.104   qw{
 	:classification
 	:ppi
 };
-
+use PPIx::Regexp        0.033;
 use Perl::MinimumVersion::Reason ();
 
-use constant REASON => 'Perl::MinimumVersion::Reason';
-
-use vars qw{$VERSION @ISA @EXPORT_OK %CHECKS %MATCHES};
+our (@ISA, @EXPORT_OK, %CHECKS, @CHECKS_RV ,%MATCHES);
 BEGIN {
-	$VERSION = '1.23_03';
-
-	# Only needed for dev releases, comment out otherwise
-	$VERSION = eval $VERSION;
-
 	# Export the PMV convenience constant
 	@ISA       = 'Exporter';
 	@EXPORT_OK = 'PMV';
 
 	# The primary list of version checks
 	%CHECKS = (
+        # _stacked_labels         => version->new('5.014'),
+
+		_yada_yada_yada         => version->new('5.012'),
+		_pkg_name_version       => version->new('5.012'),
+		_postfix_when           => version->new('5.012'),
+		_perl_5012_pragmas      => version->new('5.012'),
+		_while_readdir          => version->new('5.012'),
+
 		_perl_5010_pragmas      => version->new('5.010'),
 		_perl_5010_operators    => version->new('5.010'),
 		_perl_5010_magic        => version->new('5.010'),
+		_state_declaration      => version->new('5.010'),
 
 		# Various small things
 		_bugfix_magic_errno     => version->new('5.008.003'),
+		_is_utf8                => version->new('5.008.001'),
 		_unquoted_versions      => version->new('5.008.001'),
 		_perl_5008_pragmas      => version->new('5.008'),
 		_constant_hash          => version->new('5.008'),
-		_use_base_exporter      => version->new('5.008'),
 		_local_soft_reference   => version->new('5.008'),
 		_use_carp_version       => version->new('5.008'),
+		_open_temp              => version->new('5.008'),
+		_open_scalar            => version->new('5.008'),
+		_internals_svreadonly   => version->new('5.008'),
 
 		# Included in 5.6. Broken until 5.8
 		_pragma_utf8            => version->new('5.008'),
@@ -91,6 +104,8 @@ BEGIN {
 		_three_argument_open    => version->new('5.006'),
 		_weaken                 => version->new('5.006'),
 		_mkdir_1_arg            => version->new('5.006'),
+		_exists_subr            => version->new('5.006'),
+		_sort_subref            => version->new('5.006'),
 
 		_any_qr_tokens          => version->new('5.005.03'),
 		_perl_5005_pragmas      => version->new('5.005'),
@@ -100,12 +115,21 @@ BEGIN {
 		_any_INIT_blocks        => version->new('5.005'),
 		_substr_4_arg           => version->new('5.005'),
 		_splice_negative_length => version->new('5.005'),
+		_5005_variables         => version->new('5.005'),
+		_bareword_double_colon  => version->new('5.005'),
 
 		_postfix_foreach        => version->new('5.004.05'),
 	);
+	@CHECKS_RV = ( #subs that return version
+	    '_feature_bundle','_regex','_each_argument','_binmode_2_arg',
+        '_scheduled_blocks',
+	);
 
 	# Predefine some indexes needed by various check methods
 	%MATCHES = (
+		_perl_5012_pragmas => {
+			deprecate => 1,
+		},
 		_perl_5010_pragmas => {
 			mro     => 1,
 			feature => 1,
@@ -123,6 +147,7 @@ BEGIN {
 			threads           => 1,
 			'threads::shared' => 1,
 			sort              => 1,
+			encoding          => 1,
 		},
 		_perl_5006_pragmas => {
 			warnings             => 1, #may be ported into older version
@@ -268,7 +293,7 @@ sub minimum_reason {
 }
 
 sub default_reason {
-	Perl::MinimumVersion->new(
+	Perl::MinimumVersion::Reason->new(
 		rule    => 'default',
 		version => $_[0]->{default},
 		element => undef,
@@ -321,7 +346,7 @@ sub _minimum_explicit_version {
 	my $element = undef;
 	foreach my $include ( @$explicit ) {
 		my $version = version->new($include->version);
-		if ( $version > $max or not $element ) {
+		if ( not $element or $version > $max ) {
 			$max     = $version;
 			$element = $include;
 		}
@@ -375,7 +400,7 @@ sub minimum_syntax_reason {
 		$limit = version->new("$limit");
 	}
 	if ( defined $self->{syntax} ) {
-		if ( $self->{syntax} >= $limit ) {
+		if ( !defined($limit) or $self->{syntax}->version >= $limit ) {
 			# Previously discovered minimum is what they want
 			return $self->{syntax};
 		}
@@ -399,32 +424,75 @@ sub minimum_syntax_reason {
 	return '';
 }
 
+#for Perl::Critic::Policy::Compatibility::PerlMinimumVersionAndWhy
+sub _set_checks2skip {
+	my $self = shift;
+	my $list = shift;
+	$self->{_checks2skip} = $list;
+}
+sub _set_collect_all_reasons {
+	my $self = shift;
+	my $value = shift;
+	$value = 1 unless defined $value;
+	$self->{_collect_all_reasons} = $value;
+}
+
 sub _minimum_syntax_version {
 	my $self   = shift;
 	my $filter = shift || $self->{default};
 
+	my %checks2skip;
+	@checks2skip{ @{ $self->{_checks2skip} || [] } } = ();
+
+	my %rv_result;
+	my $current_reason;
+	foreach my $rule ( @CHECKS_RV ) {
+		next if exists $checks2skip{$rule};
+		my ($v, $obj) = $self->$rule();
+		$v = version->new($v);
+		if ( $v > $filter ) {
+			$current_reason = Perl::MinimumVersion::Reason->new(
+				rule    => $rule,
+				version => $v,
+				element => _INSTANCE($obj, 'PPI::Element'),
+			);
+		    if ($self->{_collect_all_reasons}) {
+				push @{ $self->{_all_reasons} }, $current_reason;
+			} else {
+				$filter = $v;
+			}
+	    }
+	}
+
+
 	# Always check in descending version order.
 	# By doing it this way, the version of the first check that matches
 	# is also the version of the document as a whole.
 	my @rules = sort {
 		$CHECKS{$b} <=> $CHECKS{$a}
 	} grep {
-		$CHECKS{$_} > $filter
+	    not(exists $checks2skip{$_}) and $CHECKS{$_} > $filter
 	} keys %CHECKS;
 
 	foreach my $rule ( @rules ) {
 		my $result = $self->$rule() or next;
 
 		# Create the result object
-		return Perl::MinimumVersion::Reason->new(
+		my $reason = Perl::MinimumVersion::Reason->new(
 			rule    => $rule,
 			version => $CHECKS{$rule},
 			element => _INSTANCE($result, 'PPI::Element'),
 		);
+		if ($self->{_collect_all_reasons}) {
+			push @{ $self->{_all_reasons} }, $current_reason;
+		} else {
+			return $reason;
+		}
+
 	}
 
 	# Found nothing of interest
-	return '';
+	return $current_reason || '';
 }
 
 =pod
@@ -505,10 +573,377 @@ sub version_markers {
 
 
 
-
 #####################################################################
 # Version Check Methods
 
+my %feature =
+(
+    'state'             => '5.10',
+    'switch'            => '5.10',
+    'unicode_strings'   => '5.14',
+    'unicode_eval'      => '5.16',
+    'evalbytes'         => '5.16',
+    'current_sub'       => '5.16',
+    'array_base'        => '5.16', #defined only in 5.16
+    'fc'                => '5.16',
+    'lexical_subs'      => '5.18',
+);
+my $feature_regexp = join('|', keys %feature);
+
+#:5.14 means same as :5.12, but :5.14 is not defined in feature.pm in perl 5.12.
+sub _feature_bundle {
+    my @versions;
+    my ($version, $obj);
+	shift->Document->find( sub {
+		$_[1]->isa('PPI::Statement::Include') or return '';
+		$_[1]->pragma eq 'feature'            or return '';
+		my @child = $_[1]->schildren;
+		my @args = @child[1..$#child]; # skip 'use', 'feature' and ';'
+		foreach my $arg (@args) {
+		    my $v = 0;
+		    $v = $1 if ($arg->content =~ /:(5\.\d+)(?:\.\d+)?/);
+		    $v = max($v, $feature{$1}) if ($arg->content =~ /\b($feature_regexp)\b/);
+			#
+			if ($v and $v > ($version || 0) ) {
+				$version = $v;
+				$obj = $_[1];
+			}
+		}
+		return '';
+	} );
+	return (defined($version)?"$version.0":undef, $obj);
+}
+
+my %SCHEDULED_BLOCK =
+(
+    'INIT'      => '5.006',
+    'CHECK'     => '5.006002',
+    'UNITCHECK' => '5.010',
+);
+
+sub _scheduled_blocks
+{
+    my @versions;
+    my ($version, $obj);
+
+	shift->Document->find( sub {
+		$_[1]->isa('PPI::Statement::Scheduled') or return '';
+        ($_[1]->children)[0]->isa('PPI::Token::Word') or return '';
+        my $function = (($_[1]->children)[0])->content;
+        exists( $SCHEDULED_BLOCK{ $function }) or return '';
+
+        my $v = $SCHEDULED_BLOCK{ ($_[1]->children)[0]->content };
+        if ($v and $v > ($version || 0) ) {
+            $version = $v;
+            $obj = $_[1];
+        }
+
+		return '';
+	} );
+	return (defined($version) ? $version : undef, $obj);
+}
+
+sub _regex {
+    my @versions;
+    my ($version, $obj);
+	shift->Document->find( sub {
+	    return '' unless
+			grep { $_[1]->isa($_) }
+			qw/PPI::Token::QuoteLike::Regexp PPI::Token::Regexp::Match PPI::Token::Regexp::Substitute/;
+			my $re = PPIx::Regexp->new( $_[1] );
+        	my $v = $re->perl_version_introduced;
+			if ($v and $v > ($version || 0) ) {
+				$version = $v;
+				$obj = $_[1];
+			}
+		return '';
+	} );
+	$version = undef if ($version and $version eq '5.000');
+	return ($version, $obj);
+}
+
+sub _each_argument {
+    my ($version, $obj);
+	shift->Document->find( sub {
+		$_[1]->isa('PPI::Token::Word') or return '';
+		$_[1]->content =~ '^(each|keys|values)$'  or return '';
+		return '' if is_method_call($_[1]);
+		my $next = $_[1]->snext_sibling;
+		$next = $next->schild(0)->schild(0) if $next->isa('PPI::Structure::List');
+		if($next->isa('PPI::Token::Cast')) {
+			if($next->content eq '@' && 5.012 > ($version || 0)) {
+				$version = 5.012;
+				$obj = $_[1]->parent;
+			} elsif($next->content eq '$' && 5.014 > ($version || 0)) {
+				$version = 5.014;
+				$obj = $_[1]->parent;
+			}
+		} elsif($next->isa('PPI::Token::Symbol')) {
+			if($next->raw_type eq '@' && 5.012 > ($version || 0)) {
+				$version = 5.012;
+				$obj = $_[1]->parent;
+			} elsif($next->raw_type eq '$' && 5.014 > ($version || 0)) {
+				$version = 5.014;
+				$obj = $_[1]->parent;
+			}
+		} elsif($next->isa('PPI::Token::Operator')) { # % $a
+			return '';
+		} elsif($_[1]->parent->isa('PPI::Statement::Sub')) { # sub each|keys|values
+			return '';
+		} else { # function call or other should be reference
+			if(5.014 > ($version || 0)) {
+				$version = 5.014;
+				$obj = $_[1]->parent;
+			}
+		}
+		return 1 if ($version and $version == 5.014);
+		return '';
+	} );
+	return (defined($version)?"$version":undef, $obj);
+}
+
+#Is string (first argument) in list (other arguments)
+sub _str_in_list {
+	my $str = shift;
+	foreach my $s (@_) {
+		return 1 if $s eq $str;
+	}
+	return 0;
+}
+
+
+sub _binmode_2_arg {
+    my ($version, $obj);
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'binmode'       or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments == 2 ) {
+		    my $arg2=$arguments[1][0];
+			if ( $arg2->isa('PPI::Token::Quote')) { #check second argument
+				my $str = $arg2->string;
+				$str =~ s/^\s+//s;
+				$str =~ s/\s+$//s;
+				$str =~ s/:\s+/:/g;
+				if ( !_str_in_list( $str => qw/:raw :crlf/) and $str !~ /[\$\@\%]/) {
+            		$version = 5.008;
+		            $obj = $main_element;
+					return 1;
+				}
+			}
+			if (!$version) {
+        	    $version = 5.006;
+	            $obj = $main_element;
+	        }
+		}
+		return '';
+	} );
+	return ($version, $obj);
+}
+
+
+
+#http://perldoc.perl.org/functions/readdir.html
+#while(readdir $dh) requires perl 5.12
+sub _while_readdir {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Word') or return '';
+		$_[1]->content eq 'while' or return '';
+		return '' if is_hash_key($_[1]);
+		return '' if is_method_call($_[1]);
+		my $e1 = $_[1]->next_sibling or return '';
+		if ($e1->isa('PPI::Structure::Condition')) { #while ()
+			my @children = $e1->children;
+			$e1 = $children[0];
+		}
+		$e1->isa('PPI::Statement::Expression') or return '';
+		my @children = $e1->schildren;
+	    $e1 = $children[0];
+
+		$e1->isa('PPI::Token::Word') or return '';
+		$e1->content eq 'readdir' or return '';
+		return 1 if @children == 1; #incorrect call
+		return '' if @children > 2; #not only readdir
+		$e1 = $children[1];
+		$e1->isa('PPI::Structure::List') or $e1->isa('PPI::Token::Symbol') or return '';
+		#readdir($dh) or readdir $dh
+
+		return 1;
+	} );
+}
+
+sub _perl_5012_pragmas {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Include')
+		and
+		$MATCHES{_perl_5012_pragmas}->{$_[1]->pragma}
+	} );
+}
+
+sub _sort_subref {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Word') or return '';
+		$_[1]->content eq 'sort' or return '';
+		is_function_call($_[1]) or return '';
+		my $e1 = $_[1]->next_sibling;
+		$e1->isa('PPI::Token::Whitespace') or return '';
+		$e1 = $e1->next_sibling;
+		_get_resulting_sigil($e1) || '' eq '$' or return '';
+		$e1 = $e1->next_sibling;
+		$e1->isa('PPI::Token::Whitespace') or return '';
+		$e1 = $e1->next_sibling;
+		$e1->isa('PPI::Token::Word') or $e1->isa('PPI::Token::Symbol')
+			or $e1->isa('PPI::Token::Cast') or $e1->isa('PPI::Structure::List') or return '';
+		return 1;
+	} );
+}
+
+sub _open_temp {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement') or return '';
+		my @children = $_[1]->children;
+		#@children >= 7                or return '';
+		my $main_element = $children[0];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'open'       or return '';
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments == 3 and scalar(@{$arguments[2]}) == 1) {
+		    my $arg3 = $arguments[2][0];
+		    if ($arg3->isa('PPI::Token::Word') and $arg3->content eq 'undef') {
+				return 1;
+			}
+		}
+		return '';
+	} );
+}
+
+sub _open_scalar {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement') or return '';
+		my @children = $_[1]->children;
+		#@children >= 7                or return '';
+		my $main_element = $children[0];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'open'       or return '';
+		my @arguments = parse_arg_list($main_element);
+		if ( scalar @arguments == 3) {
+		    my $arg3 = $arguments[2][0];
+		    if ($arg3->isa('PPI::Token::Cast') and $arg3->content eq '\\') {
+				return 1;
+			}
+		}
+		return '';
+	} );
+}
+
+# exists(&subr) new in 5.6.0 #
+sub _exists_subr {
+	my ($pmv) = @_;
+	$pmv->Document->find_first(sub {
+		my ($document, $elem) = @_;
+		if ($elem->isa('PPI::Token::Word')
+			&& $elem eq 'exists'
+			&& is_function_call($elem)
+			&& ($elem = first_arg($elem))
+			&& (_get_resulting_sigil($elem) || '') eq '&') {
+				return 1;
+		} else {
+			return 0;
+		}
+	});
+}
+
+sub _get_resulting_sigil {
+	my $elem = shift;
+	if ($elem->isa('PPI::Token::Cast')) {
+		return $elem->content;
+	} elsif ($elem->isa('PPI::Token::Symbol')) {
+		return $elem->symbol_type;
+	} else {
+		return undef;
+	}
+}
+
+
+sub _postfix_when {
+	shift->Document->find_first( sub {
+		my $main_element=$_[1];
+		$main_element->isa('PPI::Token::Word') or return '';
+		$main_element->content eq 'when'    or return '';
+		return '' if is_hash_key($main_element);
+		return '' if is_method_call($main_element);
+		return '' if is_subroutine_name($main_element);
+		return '' if is_included_module_name($main_element);
+		return '' if is_package_declaration($main_element);
+		my $stmnt = $main_element->statement();
+		return '' if !$stmnt;
+		return '' if $stmnt->isa('PPI::Statement::When');
+		return 1;
+	} );
+}
+
+sub _yada_yada_yada {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Operator')
+		and $_[1]->content eq '...'  or return '';
+		my @child = $_[1]->parent->schildren;
+		@child == 1 and return 1;
+		if (@child == 2) {
+			$child[1]->isa('PPI::Token::Structure')
+		}
+	} );
+}
+
+sub _state_declaration {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Variable')
+        and ($_[1]->children)[0]->isa('PPI::Token::Word')
+        and ($_[1]->children)[0]->content eq 'state'
+	} );
+}
+
+sub _stacked_labels {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Compound') || return '';
+		$_[1]->schild(0)->isa('PPI::Token::Label') || return '';
+
+		my $next = $_[1]->snext_sibling || return '';
+
+        if (   $next->isa('PPI::Statement::Compound')
+            && $next->schild(0)->isa('PPI::Token::Label')) {
+            return 1;
+        }
+
+        0;
+    } );
+}
+
+sub _internals_svreadonly {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement')
+        and ($_[1]->children)[0]->isa('PPI::Token::Word')
+        and ($_[1]->children)[0]->content eq 'Internals::SvREADONLY'
+	} );
+}
+
+sub _pkg_name_version {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Statement::Package') or return '';
+		my @child = $_[1]->schildren();
+		$child[0]->isa('PPI::Token::Word')    or return '';
+		$child[0]->content eq 'package'       or return '';
+		$child[1]->isa('PPI::Token::Word')    or return '';
+		$child[2]->isa('PPI::Token::Number')  or return '';
+		return 1;
+	} );
+}
+
 sub _perl_5010_pragmas {
 	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Statement::Include')
@@ -519,7 +954,7 @@ sub _perl_5010_pragmas {
 
 sub _perl_5010_operators {
 	shift->Document->find_first( sub {
-		$_[1]->isa('PPI::Token::Magic')
+		$_[1]->isa('PPI::Token::Operator')
 		and
 		$MATCHES{_perl_5010_operators}->{$_[1]->content}
 	} );
@@ -527,9 +962,9 @@ sub _perl_5010_operators {
 
 sub _perl_5010_magic {
 	shift->Document->find_first( sub {
-		$_[1]->isa('PPI::Token::Operator')
+		$_[1]->isa('PPI::Token::Magic')
 		and
-		$MATCHES{_perl_5010_magic}->{$_[1]->content}
+		$MATCHES{_perl_5010_magic}->{$_[1]->symbol}
 	} );
 }
 
@@ -541,19 +976,29 @@ sub _perl_5008_pragmas {
 	} );
 }
 
-# FIXME: Needs to be upgraded to return something
+# 5.8.3: Reading $^E now preserves $!. Previously, the C code implementing $^E did not preserve errno, so reading $^E could cause errno and therefore $! to change unexpectedly.
 sub _bugfix_magic_errno {
 	my $Document = shift->Document;
+	my $element = $Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Magic')
+		and
+		$_[1]->symbol eq '$^E'
+	} ) || return undef;
+	#$^E is more rare than $!, so search for it first and return it
 	$Document->find_any( sub {
 		$_[1]->isa('PPI::Token::Magic')
 		and
-		$_[1]->content eq '$^E'
-	} )
-	and
-	$Document->find_any( sub {
-		$_[1]->isa('PPI::Token::Magic')
-		and
-		$_[1]->content eq '$!'
+		$_[1]->symbol eq '$!'
+	} ) || return '';
+	return $element;
+}
+
+# utf8::is_utf requires 5.8.1 unlike the rest of utf8
+sub _is_utf8 {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Word') or return '';
+		$_[1] eq 'utf8::is_utf'        or return '';
+		return 1;
 	} );
 }
 
@@ -626,18 +1071,14 @@ sub _any_our_variables {
 
 sub _any_binary_literals {
 	shift->Document->find_first( sub {
-		$_[1]->isa('PPI::Token::Number')
-		and
-		$_[1]->{_subtype}
-		and
-		$_[1]->{_subtype} eq 'binary'
-	} );	
+		$_[1]->isa('PPI::Token::Number::Binary')
+	} );
 }
 
 sub _any_version_literals {
 	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Token::Number::Version')
-	} );	
+	} );
 }
 
 
@@ -645,7 +1086,7 @@ sub _magic_version {
 	shift->Document->find_first( sub {
 		$_[1]->isa('PPI::Token::Magic')
 		and
-		$_[1]->content eq '$^V'
+		$_[1]->symbol eq '$^V'
 	} );
 }
 
@@ -717,19 +1158,6 @@ sub _any_INIT_blocks {
 	} );
 }
 
-# use base 'Exporter' doesn't work reliably everywhere until 5.008
-sub _use_base_exporter {
-	shift->Document->find_first( sub {
-		$_[1]->isa('PPI::Statement::Include')
-		and
-		$_[1]->module eq 'base'
-		and
-		# Add the "not colon" characters to avoid accidentally
-		# colliding with any other Exporter-named modules
-		$_[1]->content =~ /[^:]\bExporter\b[^:]/
-	} );
-}
-
 # You can't localize a soft reference
 sub _local_soft_reference {
 	shift->Document->find_first( sub {
@@ -770,10 +1198,10 @@ sub _use_carp_version {
 
 sub _three_argument_open {
 	shift->Document->find_first( sub {
-		$_[1]->isa('PPI::Statement')  or return '';
-		my @children=$_[1]->children;
+		$_[1]->isa('PPI::Statement') or return '';
+		my @children = $_[1]->children;
 		#@children >= 7                or return '';
-		my $main_element=$children[0];
+		my $main_element = $children[0];
 		$main_element->isa('PPI::Token::Word') or return '';
 		$main_element->content eq 'open'       or return '';
 		my @arguments = parse_arg_list($main_element);
@@ -784,12 +1212,11 @@ sub _three_argument_open {
 	} );
 }
 
-
 sub _substr_4_arg {
 	shift->Document->find_first( sub {
 		my $main_element=$_[1];
 		$main_element->isa('PPI::Token::Word') or return '';
-		$main_element->content eq 'substr'       or return '';
+		$main_element->content eq 'substr'     or return '';
 		return '' if is_hash_key($main_element);
 		return '' if is_method_call($main_element);
 		return '' if is_subroutine_name($main_element);
@@ -825,7 +1252,7 @@ sub _splice_negative_length {
 	shift->Document->find_first( sub {
 		my $main_element=$_[1];
 		$main_element->isa('PPI::Token::Word') or return '';
-		$main_element->content eq 'splice'       or return '';
+		$main_element->content eq 'splice'     or return '';
 		return '' if is_hash_key($main_element);
 		return '' if is_method_call($main_element);
 		return '' if is_subroutine_name($main_element);
@@ -856,7 +1283,7 @@ sub _postfix_foreach {
 	shift->Document->find_first( sub {
 		my $main_element=$_[1];
 		$main_element->isa('PPI::Token::Word') or return '';
-		$main_element->content eq 'foreach'       or return '';
+		$main_element->content eq 'foreach'    or return '';
 		return '' if is_hash_key($main_element);
 		return '' if is_method_call($main_element);
 		return '' if is_subroutine_name($main_element);
@@ -895,7 +1322,22 @@ sub _weaken {
 	} );
 }
 
+sub _5005_variables {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Magic')
+		and
+		($_[1]->symbol eq '$!' or $_[1]->symbol eq '$^R')
+	} );
+}
 
+#added in 5.5
+sub _bareword_double_colon {
+	shift->Document->find_first( sub {
+		$_[1]->isa('PPI::Token::Word')
+		and
+		$_[1]->content =~ /::$/
+	} );
+}
 
 
 
@@ -926,15 +1368,15 @@ sub _max {
 
 	# Filter and prepare for a Schwartian maximum
 	my @valid = map {
-		[ $_, $_->isa(REASON) ? $_->version : $_ ]
+		[ $_, $_->isa('Perl::MinimumVersion::Reason') ? $_->version : $_ ]
 	} grep {
-		_INSTANCE($_, REASON)
+		_INSTANCE($_, 'Perl::MinimumVersion::Reason')
 		or
 		_INSTANCE($_, 'version')
 	} @_ or return '';
 
 	# Find the maximum
-	my $max = shift @valid; 
+	my $max = shift @valid;
 	foreach my $it ( @valid ) {
 		$max = $it if $it->[1] > $max->[1];
 	}
@@ -970,6 +1412,11 @@ B<Write the explicit version checker>
 
 B<Write the recursive module descend stuff>
 
+_while_readdir for postfix while without brackets
+
+B<Check for more 5.12 features (currently only detecting
+C<package NAME VERSION;>, C<...>, and C<use feature ':5.12'>)>
+
 =head1 SUPPORT
 
 All bugs should be filed via the CPAN bug tracker at
@@ -984,11 +1431,21 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
+L<perlver> - the command-line script for running C<Perl::MinimumVersion>
+on your code.
+
+L<Perl::MinimumVersion::Fast> - another module which does the same thing.
+It's a lot faster, but only supports Perl 5.8.1+.
+
 L<http://ali.as/>, L<PPI>, L<version>
 
+=head1 REPOSITORY
+
+L<https://github.com/neilbowers/Perl-MinimumVersion>
+
 =head1 COPYRIGHT
 
-Copyright 2005 - 2010 Adam Kennedy.
+Copyright 2005 - 2014 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
