@@ -1,5 +1,5 @@
 package Perl::MinimumVersion::Reason;
-
+$Perl::MinimumVersion::Reason::VERSION = '1.38_01';
 # Simple abstraction for a syntax limitation.
 # It contains the limiting version, the rule responsible, and the
 # PPI element responsible for the limitation (if any).
@@ -8,14 +8,6 @@ use 5.006;
 use strict;
 use warnings;
 
-use vars qw{$VERSION};
-BEGIN {
-	$VERSION = '1.23_03';
-
-	# Only needed for dev releases, comment out otherwise
-	$VERSION = eval $VERSION;
-}
-
 sub new {
 	my $class = shift;
 	return bless { @_ }, $class;
@@ -33,4 +25,8 @@ sub element {
 	$_[0]->{element};
 }
 
+sub explanation {
+	$_[0]->{explanation};
+}
+
 1;
